function theta = VARestimationLasso(Y,X,p)
%_____________________________________________________________________________________________________________________
% VARestimationLasso estimates the the Matlab lasso for a VAR(p).
%
%   PARAMETERS:
%   Y is an n-vector with the data of the dependent variable
%   X is a vector indicating which variables are included in the model
%   p is maximum number of lags allowed in the general VAR(p) model
%
%   theta is the vector of lasso parameter estimates
%_____________________________________________________________________________________________________________________
%
% 1. Initialize a few variables
%_____________________________________________________________________________________________________________________
[g,n]   = size(Y);   % g is the number of dependent variables
                     % n is the length of time series
 X   = X(X>0);    % Drop the zeros
 px  = length(X); % Number of regressors (including constants)
 R   = zeros(g*(1+p*g), px);
 idx = sub2ind(size(R), X, 1:px);
 R(idx) = 1;
% 
YY      = [mean(Y,2)*ones(1,p),Y];   % Dependent variable; presample = unconditional mean
%
% Construct the Z matrix
YY      = YY(:,end:-1:1)./(std(YY,0,2)*ones(1,length(YY))); % Invert the order of the time series
YY      = YY(:)*ones(1,n); % Vectorize the matrix of dependent variables and replicate it n times
Bidx    = (1:g*p)'*ones(1,n+1) + ones(g*p,1)*(0:g:g*n); % Construct the index of lags at each time t
ZZ      = YY(Bidx); % Define the Z matrix of lags
ZZ      = ZZ(:,end:-1:1); % Invert back the order of time series
ZZ      = [ones(1,size(ZZ,2)); ZZ]; % Add the first line of constants
Z       = ZZ(:,1:end-1); % Drop the last line, which corresponds to Z_n. Keep it in ZZ for later
%_____________________________________________________________________________________________________________________
%
% 2. Parameter estimation
%_____________________________________________________________________________________________________________________
y1       = Y./(std(Y,0,2)*ones(1,length(Y))); % Vectorise the matrix of dependent variables
y       = y1(:); % Vectorise the matrix of dependent variables
ZZ1     = Z*Z';
theta10 = kron(ZZ1,sparse(eye(g)));
theta11 = kron(Z,sparse(eye(g)));
theta1  = (R'*theta10*R)\(R'*theta11*y); % Estimate without VC matrix
b       = R*theta1;
BETA    = reshape(b,g,1+g*p);
BZ      = BETA*Z;
U       = Y-BZ;
S       = U*U'/n; % Residuals VC matrix
S1      = inv(S); % Inverse of the residual VC matrix
S12     = S1^(1/2);
WW      = R'*kron(Z, S12);
WY      = kron(sparse(eye(size(Z,2))),S12) * y;
theta   = lasso(WW',WY);