function [l,dl] = VARloglik(theta,Y,R,p)
%_____________________________________________________________________________________________________________________
% Computes the quasi log-likelihood of the subset model of a VAR(P) implied by the
% constraint matrix R.
%
%   PARAMETERS:
%   theta is the vector of QMLE
%   Y is an n-vector with the data of the dependent variable
%   R is the matrix of constraints
%   p is maximum number of lags allowed in the general VAR(p) model
%
%   l is the quasi log-likelihood
%   dl is the gradient of l
%_____________________________________________________________________________________________________________________
%
% 1. Initialize a few variables
%_____________________________________________________________________________________________________________________
[g,n] = size(Y);   % g is the number of dependent variables
                   % n is the length of time series
YY = [mean(Y,2)*ones(1,p),Y];   % Dependent variable; presample = unconditional mean
% Construct the Z matrix
YY = YY(:,end:-1:1); % Invert the order of the time series
YY = YY(:)*ones(1,n); % Vectorize the matrix of dependent variables and replicate it n times
B  = (1:g*p)'*ones(1,n+1) + ones(g*p,1)*(0:g:g*n); % Construct the index of lags at each time t
ZZ = YY(B); % Define the Z matrix of lags
ZZ = ZZ(:,end:-1:1); % Invert back the order of time series
ZZ = [ones(1,size(ZZ,2)); ZZ]; % Add the first line of constants
Z  = ZZ(:,1:end-1); % Drop the last line, which corresponds to Z_n. Keep it in ZZ for later
%_____________________________________________________________________________________________________________________
%
% 2. Parameter estimation
%_____________________________________________________________________________________________________________________
y       = Y(:); % Vectorise the matrix of dependent variables
ZZ      = Z*Z';
theta10 = kron(ZZ,sparse(eye(g)));
theta11 = kron(Z,sparse(eye(g)));
theta1  = (R'*theta10*R)\(R'*theta11*y); % Estimate without VC matrix
b1      = R*theta1;
C1      = reshape(b1,g,1+g*p);
U1      = Y-C1*Z;
S       = U1*U1'/n; % Residuals VC matrix
S1      = inv(S); % Inverse of the residual VC matrix
B0      = kron(Z,S1);
b       = R*theta;
C       = reshape(b,g,1+g*p);
U       = Y-C*Z;
u       = U(:);
l1      = kron(sparse(eye(n)),S1);
l       = -0.5 * u' * l1 * u / n;
dl      = u'*B0'*R;
l       = -l; % Change sign to the log-likelihood for the minimization algorithm
dl = -dl'; % Change sign and transpose to have a column vector of first derivatives